==> linux-gpib-1.1b.base.tgz: linux-gpib/driver/sys/osinterrupt.c <==
#include <ibsys.h>


/*
 * There is a Problem with Linux Semaphores: 
 *      - enabling Interrupts
 *      - call to down()
 *
 * could produce spurious segmentation faults, because up() is called
 * within this interrupt just before down() has been called
 * then wait will be called with empty queue.
 *
 * I worked over this code again to protect from such conditions
 * hope it works. :-)
 *
 *
 *
 */

extern uint8 ibirq;

static int IRQ_mask;

#if USEINTS
/*
 * GPIB interrupt service routine -- fast and simple
 */
IBLCL void ibintr(int irq)
{
int s;

#if DEBUG
	if (dbgMask & DBG_INTR)
	        printk("GPIB INTERRUPT! semaphore id = %d\n", espsemid.count);
#endif
#ifdef NIAT
	GPIBout(imr3, 0);		                /* disable interrupts */
#endif
#ifdef NIPCII
	GPIBout(imr2, 0);
#endif
#ifdef NIPCIIa

	GPIBout(imr2, 0);
#if DEBUG
        if(dbgMask & DBG_INTR) printk("GPIB-IRQ nipcii\n");
#endif
        /* reset shared interrupt circuit */
	outb(0xff , ( 0x2f0 | ibirq ) );

#endif
	    up(&espsemid); /* wake up sleeping process */
}


/*
 * Wait for GPIB or timer interrupt.  Semaphore will be posted by
 * either ibintr or ibtmintr.
 */



IBLCL osWaitForInt(imr3mask)
int imr3mask;
{
        struct wait_queue wait = {current, NULL}; /*@*/

	DBGin("osWaitForInt");
	if (espsemid.count <= 0) {
	/*
	 *	If semaphore not already available, enable
	 *	requested interrupts and wait until it is...
	 */

	  espsemid.count = 1;
          down(&espsemid); /* spurious interrups calling up() while enable irq's ? */

          /* now it's time to enable board interrupts */
#ifdef NIAT    
	        DBGprint(DBG_DATA, ("imr3mask=0x%x  ", imr3mask));		
                GPIBout(imr3, imr3mask);
#endif
#ifdef NIPCII
                DBGprint(DBG_DATA, ("imr2mask=0x%x  ", imr3mask));		
		GPIBout(imr2, imr3mask); /* sorry */	        
#endif
		/* now push process to sleep */
		down(&espsemid);
		up(&espsemid); /* wakeup process */
	}
        else	{ 
	  espsemid.count = 0;
          espsemid.wait  = NULL;
	}
	DBGout();
}
#endif

==> linux-gpib-1.3a.base.tgz: linux-gpib/driver/sys/osinterrupt.c <==
#include <ibsys.h>


/*
 * There is a Problem with Linux Semaphores: 
 *      - enabling Interrupts
 *      - call to down()
 *
 * could produce spurious segmentation faults, because up() is called
 * within this interrupt just before down() has been called
 * then wait will be called with empty queue.
 *
 * I worked over this code again to protect from such conditions
 * hope it works. :-)
 *
 *
 *
 */

extern uint8 ibirq;

static int IRQ_mask;

#if USEINTS
/*
 * GPIB interrupt service routine -- fast and simple
 */
IBLCL void ibintr(int irq, struct pt_regs *registerp )
{
int s;

#if DEBUG
	if (dbgMask & DBG_INTR)
	        printk("GPIB INTERRUPT! semaphore id = %d\n", espsemid.count);
#endif
#ifdef NIAT
	GPIBout(imr3, 0);		                /* disable interrupts */
#endif
#ifdef NIPCII
	GPIBout(imr2, 0);
#endif
#ifdef NIPCIIa

	GPIBout(imr2, 0);
#if DEBUG
        if(dbgMask & DBG_INTR) printk("GPIB-IRQ nipcii\n");
#endif
        /* reset shared interrupt circuit */
	outb(0xff , ( 0x2f0 | ibirq ) );

#endif
	    up(&espsemid); /* wake up sleeping process */
}


/*
 * Wait for GPIB or timer interrupt.  Semaphore will be posted by
 * either ibintr or ibtmintr.
 */



IBLCL osWaitForInt(imr3mask)
int imr3mask;
{
        struct wait_queue wait = {current, NULL}; /*@*/

	DBGin("osWaitForInt");
	if (espsemid.count <= 0) {
	/*
	 *	If semaphore not already available, enable
	 *	requested interrupts and wait until it is...
	 */

	  espsemid.count = 1;
          down(&espsemid); /* spurious interrups calling up() while enable irq's ? */

          /* now it's time to enable board interrupts */
#ifdef NIAT    
	        DBGprint(DBG_DATA, ("imr3mask=0x%x  ", imr3mask));		
                GPIBout(imr3, imr3mask);
#endif
#ifdef NIPCII
                DBGprint(DBG_DATA, ("imr2mask=0x%x  ", imr3mask));		
		GPIBout(imr2, imr3mask); /* sorry */	        
#endif
		/* now push process to sleep */
		down(&espsemid);
		up(&espsemid); /* wakeup process */
	}
        else	{ 
	  espsemid.count = 0;
          espsemid.wait  = NULL;
	}
	DBGout();
}
#endif
